@@ -19,10 +19,13 @@ from rich.style import Style
 from rich.markdown import Markdown
 from rich.console import Console
 from rich.live import Live
+from rich.table import Table
 from rich.prompt import Prompt
 from typing import Iterator
 from pytgpt.utils import Optimizers
 from pytgpt.utils import default_path
+from pytgpt.utils import AwesomePrompts
+from WebChatGPT.console import chat as webchatgpt
 
 getExc = lambda e: e.args[1] if len(e.args) > 1 else str(e)
 
@@ -55,6 +58,7 @@ def stream_output(
     title_generator: object = None,
     title_generator_params: dict = {},
     code_theme: str = "monokai",
+    vertical_overflow: str = "ellipsis",
 ) -> None:
     """Stdout streaming response
 
@@ -67,13 +71,14 @@ def stream_output(
         title_generator (object, optional): Function for generating title. Defaults to None.
         title_generator_params (dict, optional): Kwargs for `title_generator` function. Defaults to {}.
         code_theme (str, optional): Theme for styling codes. Defaults to `monokai`
+        vertical_overflow (str, optional): Vertical overflow behaviour on content display. Defaultss to ellipsis.
     """
     render_this = ""
     with Live(
         render_this,
         transient=transient,
         refresh_per_second=8,
-        vertical_overflow="visible",
+        vertical_overflow=vertical_overflow,
     ) as live:
         for entry in iterable:
             render_this += entry
@@ -104,9 +109,26 @@ def clear_history_file(file_path, is_true):
             logging.error(f"Failed to clear previous chat history - {getExc(e)}")
 
 
+def handle_exception(func):
+    """Safely handles cli-based exceptions and exit status-codes"""
+
+    @wraps(func)
+    def decorator(*args, **kwargs):
+        try:
+            exit_status = func(*args, **kwargs)
+        except Exception as e:
+            exit_status = False
+            logging.error(getExc(e))
+        finally:
+            exit(0 if exit_status not in (False, "") else 1)
+
+    return decorator
+
+
 class busy_bar:
     querying = None
     __spinner = (
+        (),
         ("-", "\\", "|", "/"),
         (
             "█■■■■",
@@ -183,7 +205,7 @@ class busy_bar:
 
 
 class Main(cmd.Cmd):
-    intro = f"Welcome to AI Chat in terminal. Type 'help' or '?' for usage info \n Submit any bug at {pytgpt.__repo__}/issues/new"
+    intro = f"Welcome to AI Chat in terminal. Type 'help' or 'h' for usage info \n Submit any bug at {pytgpt.__repo__}/issues/new"
     prompt = f"╭─[{getpass.getuser().capitalize()}@TGPT](v{pytgpt.__version__})\n╰─>"
 
     def __init__(
@@ -310,6 +332,18 @@ class Main(cmd.Cmd):
                     act=awesome_prompt,
                 )
 
+            elif provider == "bard":
+                from pytgpt.bard import BARD
+
+                assert (
+                    auth
+                ), f"Bard's session-id or path to bard.google.com.cookies.json file is required"
+                self.bot = BARD(
+                    auth=auth,
+                    proxy=proxies,
+                    timeout=timeout,
+                )
+
             else:
                 raise NotImplementedError(
                     f"The provider `{provider}` is not yet implemented."
@@ -323,6 +357,8 @@ class Main(cmd.Cmd):
         self.color = "cyan"
         self.code_theme = "monokai"
         self.quiet = quiet
+        self.vertical_overflow = "ellipsis"
+        self.disable_stream = False
 
     def output_bond(
         self,
@@ -365,11 +401,52 @@ class Main(cmd.Cmd):
                 json.dump(text, fh, indent=4)
             click.secho(f"Successfuly saved to `{save_to}`", fg="green")
 
+    def do_h(self, line):
+        """Show help info in tabular form"""
+        table = Table(
+            title="Help info",
+            show_lines=True,
+        )
+        table.add_column("No.", style="white", justify="center")
+        table.add_column("Command", style="yellow", justify="left")
+        table.add_column("Function", style="cyan")
+        command_methods = [
+            getattr(self, method)
+            for method in dir(self)
+            if callable(getattr(self, method)) and method.startswith("do_")
+        ]
+        command_methods.append(self.default)
+        command_methods.reverse()
+        for no, method in enumerate(command_methods):
+            table.add_row(
+                str(no + 1),
+                method.__name__[3:] if not method == self.default else method.__name__,
+                method.__doc__,
+            )
+        Console().print(table)
+
     @busy_bar.run("Settings saved")
     def do_settings(self, line):
         """Configre settings"""
+        self.prettify = click.confirm(
+            "\nPrettify markdown response", default=self.prettify
+        )
+        busy_bar.spin_index = click.prompt(
+            "Spin bar index [0: None, 1:/, 2:■█■■■, 3:⣻]",
+            default=busy_bar.spin_index,
+            type=click.IntRange(0, 3),
+        )
+        self.color = click.prompt("Response stdout font color", default=self.color)
+        self.code_theme = Prompt.ask(
+            "Enter code_theme", choices=rich_code_themes, default=self.code_theme
+        )
+        self.vertical_overflow = Prompt.ask(
+            "\nVertical overflow behaviour",
+            choices=["ellipsis", "visible", "crop"],
+            default=self.vertical_overflow,
+        )
         self.bot.max_tokens_to_sample = click.prompt(
-            "Maximum tokens to sample",
+            "\nMaximum tokens to sample",
             type=click.INT,
             default=self.bot.max_tokens_to_sample,
         )
@@ -389,18 +466,6 @@ class Main(cmd.Cmd):
         self.bot.model = click.prompt(
             "Model name", type=click.STRING, default=self.bot.model
         )
-        self.code_theme = Prompt.ask(
-            "Enter code_theme", choices=rich_code_themes, default=self.code_theme
-        )
-        self.prettify = click.confirm(
-            "\nPrettify markdown response", default=self.prettify
-        )
-        busy_bar.spin_index = click.prompt(
-            "Spin bar index [0:/, 1:■█■■■, 2:⣻]",
-            default=busy_bar.spin_index,
-            type=click.IntRange(0, 2),
-        )
-        self.color = click.prompt("Response stdout font color", default=self.color)
 
     @busy_bar.run(help="System error")
     def do_copy_this(self, line):
@@ -541,6 +606,14 @@ class Main(cmd.Cmd):
             self.bot.conversation.chat_history = fh.read()
         click.secho("Conversation loaded successfully.", fg="cyan")
 
+    def do_last_response(self, line):
+        """Show whole last response in json format"""
+        self.output_bond(
+            "Last Response",
+            self.bot.last_response,
+            is_json=True,
+        )
+
     @busy_bar.run()
     def default(self, line, exit_on_error: bool = False):
         """Chat with ChatGPT"""
@@ -550,14 +623,27 @@ class Main(cmd.Cmd):
             os.system(line[2:])
         else:
             try:
+
+                def generate_response():
+                    # Ensure response is yielded
+                    def for_stream():
+                        return self.bot.chat(line, stream=True)
+
+                    def for_non_stream():
+                        yield self.bot.chat(line, stream=False)
+
+                    return for_non_stream() if self.disable_stream else for_stream()
+
+                busy_bar.start_spinning()
+                generated_response = generate_response()
+
                 if self.quiet:
-                    generated_response = self.bot.chat(line, stream=True)
                     busy_bar.stop_spinning()
                     console_ = Console()
                     with Live(
                         console=console_,
                         refresh_per_second=16,
-                        vertical_overflow="visible",
+                        vertical_overflow=self.vertical_overflow,
                     ) as live:
                         for response in generated_response:
                             live.update(
@@ -566,8 +652,6 @@ class Main(cmd.Cmd):
                                 else response
                             )
                 else:
-                    busy_bar.start_spinning()
-                    generated_response = self.bot.chat(line, stream=True)
                     busy_bar.stop_spinning()
                     stream_output(
                         generated_response,
@@ -577,6 +661,7 @@ class Main(cmd.Cmd):
                             color=self.color,
                         ),
                         code_theme=self.code_theme,
+                        vertical_overflow=self.vertical_overflow,
                     )
             except (KeyboardInterrupt, EOFError):
                 busy_bar.stop_spinning()
@@ -608,6 +693,10 @@ class Main(cmd.Cmd):
 
 
 @click.group()
+@click.version_option(
+    pytgpt.__version__, "-v", "--version", package_name="pytgpt", prog_name="pytgpt"
+)
+@click.help_option("-h", "--help")
 def tgpt2_():
     pass
 
@@ -662,9 +751,9 @@ def tgpt2_():
 @click.option(
     "-bi",
     "--busy-bar-index",
-    help="Index of busy bar icon : [0:/, 1:■█■■■, 2:⣻]",
-    type=click.IntRange(0, 2),
-    default=2,
+    help="Index of busy bar icon : [0: None, 1:/, 2:■█■■■, 3:⣻]",
+    type=click.IntRange(0, 3),
+    default=3,
 )
 @click.option("-fc", "--font-color", help="Stdout font color")
 @click.option(
@@ -728,6 +817,21 @@ def tgpt2_():
     help="Name of LLM provider.",
     envvar="llm_provider",
 )
+@click.option(
+    "-vo",
+    "--vertical-overflow",
+    envvar="vertical_overflow",
+    help="Vertical overflow behaviour on content display",
+    type=click.Choice(["visible", "crop", "ellipsis"]),
+    default="ellipsis",
+)
+@click.option(
+    "-w",
+    "--whole",
+    is_flag=True,
+    default=False,
+    help="Disable streaming response",
+)
 @click.option(
     "-q",
     "--quiet",
@@ -741,6 +845,13 @@ def tgpt2_():
     help="Override the filepath contents",
     is_flag=True,
 )
+@click.option(
+    "-wc",
+    "--with-copied",
+    is_flag=True,
+    help="Postfix prompt with last copied text",
+)
+@click.help_option("-h", "--help")
 def interactive(
     model,
     temperature,
@@ -762,8 +873,11 @@ def interactive(
     awesome_prompt,
     proxy_path,
     provider,
+    vertical_overflow,
+    whole,
     quiet,
     new,
+    with_copied,
 ):
     """Chat with AI interactively"""
     clear_history_file(filepath, new)
@@ -789,7 +903,11 @@ def interactive(
     bot.code_theme = code_theme
     bot.color = font_color
     bot.prettify = prettify
+    bot.vertical_overflow = vertical_overflow
+    bot.disable_stream = whole
     if prompt:
+        if with_copied:
+            prompt = prompt + "\n" + clipman.get()
         bot.default(prompt)
     bot.cmdloop()
 
@@ -844,9 +962,9 @@ def interactive(
 @click.option(
     "-bi",
     "--busy-bar-index",
-    help="Index of busy bar icon : [0:/, 1:■█■■■, 2:⣻]",
-    type=click.IntRange(0, 2),
-    default=2,
+    help="Index of busy bar icon : [0: None, 1:/, 2:■█■■■, 3:⣻]",
+    type=click.IntRange(0, 3),
+    default=3,
 )
 @click.option(
     "-fc",
@@ -864,8 +982,8 @@ def interactive(
     "-w",
     "--whole",
     is_flag=True,
-    default=True,
-    help="Give response back as a whole text (Default)",
+    default=False,
+    help="Disable streaming response",
 )
 @click.option(
     "-c",
@@ -935,6 +1053,14 @@ def interactive(
     help="Name of LLM provider.",
     envvar="llm_provider",
 )
+@click.option(
+    "-vo",
+    "--vertical-overflow",
+    envvar="vertical_overflow",
+    help="Vertical overflow behaviour on content display",
+    type=click.Choice(["visible", "crop", "ellipsis"]),
+    default="ellipsis",
+)
 @click.option(
     "-q",
     "--quiet",
@@ -948,6 +1074,13 @@ def interactive(
     help="Override the filepath contents",
     is_flag=True,
 )
+@click.option(
+    "-wc",
+    "--with-copied",
+    is_flag=True,
+    help="Postfix prompt with last copied text",
+)
+@click.help_option("-h", "--help")
 def generate(
     model,
     temperature,
@@ -972,8 +1105,10 @@ def generate(
     awesome_prompt,
     proxy_path,
     provider,
+    vertical_overflow,
     quiet,
     new,
+    with_copied,
 ):
     """Generate a quick response with AI (Default)"""
     bot = Main(
@@ -994,6 +1129,11 @@ def generate(
         provider,
         quiet,
     )
+    if with_copied:
+        sep = "\n" if prompt else ""
+        prompt = prompt + sep + clipman.get()
+        assert prompt.strip() != sep, "No copied text found, issue prompt"
+
     if not prompt:
         # Let's try to read piped input
         import signal
@@ -1009,8 +1149,8 @@ def generate(
             prompt = click.get_text_stream("stdin").read()
         except TimeoutError as e:
             help_info = (
-                "Usage: tgpt generate [OPTIONS] PROMPT\n"
-                "Try 'tgpt generate --help' for help.\n"
+                "Usage: pytgpt generate [OPTIONS] PROMPT\n"
+                "Try 'pytgpt generate --help' for help.\n"
                 "Error: Missing argument 'PROMPT'.\n"
             )
             click.secho(
@@ -1028,26 +1168,189 @@ def generate(
     bot.code_theme = code_theme
     bot.color = font_color
     bot.prettify = prettify
+    bot.vertical_overflow = vertical_overflow
+    bot.disable_stream = whole
     bot.default(prompt, True)
 
 
+@tgpt2_.group()  # For awesome-prompts
+@click.help_option("-h", "--help")
+def awesome():
+    """Perform CRUD operations on awesome-prompts"""
+    pass
+
+
+@awesome.command()
+@click.option(
+    "-r",
+    "--remote",
+    help="Remote source to update from",
+    default=AwesomePrompts.awesome_prompt_url,
+)
+@click.option(
+    "-o",
+    "--output",
+    help="Path to save the prompts",
+    default=AwesomePrompts.awesome_prompt_path,
+)
+@click.option(
+    "-n", "--new", is_flag=True, help="Override the existing contents in path"
+)
+@click.help_option("-h", "--help")
+@handle_exception
+def update(remote, output, new):
+    """Update awesome-prompts from remote source."""
+    AwesomePrompts.awesome_prompt_url = remote
+    AwesomePrompts.awesome_prompt_path = output
+    AwesomePrompts().update_prompts_from_online(new)
+    click.secho(f"Prompts saved to - '{AwesomePrompts.awesome_prompt_path}'", fg="cyan")
+
+
+@awesome.command()
+@click.option(
+    "-k", "--key", required=True, type=click.STRING, help="Search keyword or index"
+)
+@click.option("-d", "--default", help="Return this value if not found", default=None)
+@click.option(
+    "--case-sensitive",
+    default=True,
+    flag_value=False,
+    help="Perform case-sensitive search",
+)
+@click.option(
+    "-f",
+    "--file",
+    type=click.Path(exists=True),
+    help="Path to existing prompts",
+    default=AwesomePrompts.awesome_prompt_path,
+)
+@click.help_option("-h", "--help")
+@handle_exception
+def search(
+    key,
+    default,
+    case_sensitive,
+    file,
+):
+    """Search for a particular awesome-prompt by key or index"""
+    AwesomePrompts.awesome_prompt_path = file
+    resp = AwesomePrompts().get_act(
+        key,
+        default=default,
+        case_insensitive=case_sensitive,
+    )
+    if resp:
+        click.secho(resp)
+    return resp != default
+
+
+@awesome.command()
+@click.option("-n", "--name", required=True, help="Prompt name")
+@click.option("-p", "--prompt", required=True, help="Prompt value")
+@click.option(
+    "-f",
+    "--file",
+    type=click.Path(exists=True),
+    help="Path to existing prompts",
+    default=AwesomePrompts.awesome_prompt_path,
+)
+@click.help_option("-h", "--help")
+@handle_exception
+def add(name, prompt, file):
+    """Add new prompt to awesome-prompt list"""
+    AwesomePrompts.awesome_prompt_path = file
+    return AwesomePrompts().add_prompt(name, prompt)
+
+
+@awesome.command()
+@click.argument("name")
+@click.option(
+    "--case-sensitive",
+    is_flag=True,
+    flag_value=False,
+    default=True,
+    help="Perform name case-sensitive search",
+)
+@click.option(
+    "-f",
+    "--file",
+    type=click.Path(exists=True),
+    help="Path to existing prompts",
+    default=AwesomePrompts.awesome_prompt_path,
+)
+@click.help_option("-h", "--help")
+@handle_exception
+def delete(name, case_sensitive, file):
+    """Delete a specific awesome-prompt"""
+    AwesomePrompts.awesome_prompt_path = file
+    return AwesomePrompts().delete_prompt(name, case_sensitive)
+
+
+@awesome.command()
+@click.option(
+    "-j",
+    "--json",
+    is_flag=True,
+    help="Display prompts in json format",
+)
+@click.option(
+    "-i",
+    "--indent",
+    type=click.IntRange(1, 20),
+    help="Json format indentation level",
+    default=4,
+)
+@click.option(
+    "-x",
+    "--index",
+    is_flag=True,
+    help="Display prompts with their corresponding indexes",
+)
+@click.option("-c", "--color", help="Prompts stdout font color")
+@click.option("-o", "--output", type=click.Path(), help="Path to save the prompts")
+@click.help_option("-h", "--help")
+def all(json, indent, index, color, output):
+    """Stdout all awesome prompts"""
+    ap = AwesomePrompts()
+    awesome_prompts = ap.all_acts if index else ap.get_acts()
+    from json import dumps
+
+    formatted_awesome_prompts = dumps(awesome_prompts)
+    if json:
+        # click.secho(formatted_awesome_prompts, fg=color)
+        rich.print_json(formatted_awesome_prompts, indent=indent)
+
+    else:
+        awesome_table = Table(show_lines=True, title="All Awesome-Prompts")
+        awesome_table.add_column("index", justify="center", style="yellow")
+        awesome_table.add_column("Act Name/Index", justify="left", style="cyan")
+        awesome_table.add_column(
+            "Prompt",
+            style=color,
+        )
+        for index, key_value in enumerate(awesome_prompts.items()):
+            awesome_table.add_row(str(index), str(key_value[0]), key_value[1])
+        rich.print(awesome_table)
+
+    if output:
+        with open(output, "w") as fh:
+            fh.write(formatted_awesome_prompts)
+
+
+tgpt2_.add_command(
+    webchatgpt, "webchatgpt"
+)  # Intergration with WebChatGPT https://github.com/Simatwa/WebChatGPT
+
+
+# @handle_exception
 def main(*args):
     """Fireup console programmically"""
     sys.argv += list(args)
     args = sys.argv
-    if "--version" in args:
-        # Lets show version here and exit
-        click.secho(f"tgpt v{pytgpt.__version__}")
-        sys.exit(0)
-    if (
-        len(args) > 1
-        and args[1] not in ["interactive", "generate"]
-        and not "--help" in args
-    ):
-        sys.argv.insert(1, "generate")  # Just a hack to make default command
-    elif (
-        len(args) == 1
-    ):  # and platform.system() == "Windows": What do you think about this?
+    if len(args) > 1 and not "-" in args[1]:
+        if len(args) > 1 and args[1] not in tgpt2_.commands.keys():
+            sys.argv.insert(1, "generate")  # Just a hack to make default command
+    elif len(args) == 1:
         sys.argv.insert(1, "interactive")
     tgpt2_()
 
